@@ -6,7 +6,7 @@ from canalyst_candas.candas import (
     ModelSet,
     Chart,
 )
-from canalyst_candas.utils import get_chart_brand_config
+from canalyst_candas.utils import get_chart_brand_config, unstack_dataframe
 import canalyst_candas.version as version
 
 __version__ = version.__version__
