@@ -284,7 +284,7 @@ def get_data_set_from_mds(
     if url is None:
         return
 
-    response = get_request(url, auth_headers)
+    response = get_request(url, auth_headers, log)
     csv_string = response.content.decode("utf-8")
     df = pd.read_csv(StringIO(csv_string))
     return df
@@ -316,31 +316,15 @@ def _get_data_set_urls_from_mds(
 ):
     """
     Get's the candascsvdata_set list from the equity model instance endpoint.
-
-    Since it's simply a wrapper around the MDS, it shouldn't require automated testing.
     """
     url = f"{mds_host}/{EQUITY_MODEL_URL.format(csin=csin, version=model_version)}"
 
-    try:
-        response = get_request(url, auth_headers)
-        response.raise_for_status()
-    except requests.RequestException as ex:
-        # log an error that the csv dataset doesn't exist
-        log.write(
-            f"Candas: Error when making a request to '{url}'. " f"Exception: {ex}."
-        )
+    response_content = get_request_json_content(url, auth_headers, log)
+
+    if not response_content:
         return []
-
-    if response.ok:
-        csv_data_set_urls = response.json().get("candascsvdata_set")
     else:
-        log.write(
-            f"Candas: Error when hitting '{url}'. "
-            f"Status code: {response.status_code}. Response: {response.text}."
-        )
-        csv_data_set_urls = []
-
-    return csv_data_set_urls
+        return response_content.get("candascsvdata_set")
 
 
 def save_guidance_csv(df, ticker, filename):
@@ -477,14 +461,20 @@ def filter_dataset(
 
 
 def send_scenario(
-    ticker, data, auth_headers, csin="", latest_version="", mds_host=settings.MDS_HOST
+    ticker,
+    data,
+    auth_headers,
+    logger: LogFile,
+    csin="",
+    latest_version="",
+    mds_host=settings.MDS_HOST,
 ):
     """
     Makes a POST request for scenario to the MDS
     """
     if csin == "" or latest_version == "":
         csin, latest_version = get_company_info_from_ticker(
-            ticker, auth_headers, mds_host
+            ticker, auth_headers, logger, mds_host
         )
     scenario_url = (
         f"{mds_host}/{SCENARIO_URL.format(csin=csin, version=latest_version)}"
@@ -535,7 +525,6 @@ def get_name_index_from_csv(ticker, config):
     # check if the model exists if so use it.
     file_ticker = ticker.split(" ")[0]
 
-    log = LogFile(default_dir=config.default_dir)
     get_excel_model(config, ticker, default_dir)
 
     path_name = f"{config.default_dir}/DATA/{file_ticker}/"
@@ -545,7 +534,6 @@ def get_name_index_from_csv(ticker, config):
     for filename in files:
         if "data" not in filename:
             cache_files.append(filename)
-    list_for = []
     for f_path in cache_files:
         if "~" in f_path or ".csv" in f_path:
             continue
@@ -578,13 +566,15 @@ def get_name_index_from_csv(ticker, config):
     return df_index
 
 
-def get_scenarios(ticker, auth_headers, mds_host=settings.MDS_HOST):
+def get_scenarios(ticker, auth_headers, logger: LogFile, mds_host=settings.MDS_HOST):
     if ticker == "":
         ticker = ticker
     url = f"{mds_host}/api/scenarios/"
-    csin, latest_version = get_company_info_from_ticker(ticker, auth_headers, mds_host)
-    response = requests.get(url, headers=auth_headers, verify=False)
-    res = response.json().get("results")
+    csin, latest_version = get_company_info_from_ticker(
+        ticker, auth_headers, logger, mds_host
+    )
+    response_content = get_request_json_content(url, auth_headers, logger)
+    res = response_content.get("results")
     list_out = []
     for i in res:
         dict_out = {}
@@ -610,14 +600,16 @@ def post_request(url, headers, json):
     return response
 
 
-def get_company_info_from_ticker(ticker, auth_headers, mds_host=settings.MDS_HOST):
+def get_company_info_from_ticker(
+    ticker, auth_headers, logger: LogFile, mds_host=settings.MDS_HOST
+):
     """
     Get CSIN and latest version for a ticker
     """
     company_url = f"{mds_host}/{CSIN_URL.format(ticker=quote_plus(ticker))}"
-    response = requests.get(company_url, headers=auth_headers, verify=False)
+    response_content = get_request_json_content(company_url, auth_headers, logger)
 
-    json = response.json().get("results")[0]
+    json = response_content.get("results")[0]
     csin: str = json.get("csin")
     latest_version: str = (
         json.get("latest_equity_model", {}).get("model_version", {}).get("name", {})
@@ -626,18 +618,20 @@ def get_company_info_from_ticker(ticker, auth_headers, mds_host=settings.MDS_HOS
     return (csin, latest_version)
 
 
-def get_csin_from_ticker(ticker, auth_headers, mds_host=settings.MDS_HOST):
+def get_csin_from_ticker(
+    ticker, auth_headers, logger: LogFile, mds_host=settings.MDS_HOST
+):
     """
     Get CSIN for ticker
     """
     company_url = f"{mds_host}/{CSIN_URL.format(ticker=quote_plus(ticker))}"
-    response = requests.get(company_url, headers=auth_headers, verify=False)
-    json = response.json().get("results")[0]
+    response_content = get_request_json_content(company_url, auth_headers, logger)
+    json = response_content.get("results")[0]
     csin = json.get("csin")
     return csin
 
 
-def get_excel_model(config, ticker="", download_dir=""):
+def get_excel_model(config, logger, ticker="", download_dir=""):
     """
     Retrieve Excel model from Web Portal
     """
@@ -649,7 +643,7 @@ def get_excel_model(config, ticker="", download_dir=""):
     else:
         default_dir = download_dir
 
-    csin = get_csin_from_ticker(ticker, auth_headers, mds_host)
+    csin = get_csin_from_ticker(ticker, auth_headers, logger, mds_host)
 
     client = GraphqlClient(endpoint=f"{wp_host}/model-workbooks")
 
@@ -735,14 +729,14 @@ def get_forecast_url(csin, latest_version, mds_host=settings.MDS_HOST):
     )
 
 
-def get_forecast_url_data(res_dict, ticker, auth_headers):
+def get_forecast_url_data(res_dict, ticker, auth_headers, log):
     list_out = []
     url = res_dict["self"]
 
-    res_loop = get_request(url, auth_headers)
+    res_loop = get_request(url, auth_headers, log)
     url = res_loop.json()["data_points"]
 
-    res_loop = get_request(url, auth_headers)
+    res_loop = get_request(url, auth_headers, log)
     # try:
     next_url = res_loop.json()["next"]
     # except:
@@ -788,7 +782,7 @@ def get_forecast_url_data(res_dict, ticker, auth_headers):
             list_out.append(df)
         url = next_url
         try:
-            res_loop = get_request(url, auth_headers)
+            res_loop = get_request(url, auth_headers, log)
             next_url = res_loop.json()["next"]
         except:
             url = None
@@ -801,10 +795,10 @@ def get_scenario_url_data(
     log = LogFile(default_dir=default_dir)
     url = res_dict["self"]
 
-    res_loop = get_request(url, auth_headers)
+    res_loop = get_request(url, auth_headers, log)
     url = res_loop.json()["data_points"]
 
-    res_loop = get_request(url, auth_headers)
+    res_loop = get_request(url, auth_headers, log)
 
     try:
         res_data = res_loop.json()["results"]
@@ -844,7 +838,7 @@ def get_scenario_url_data(
             df = pd.DataFrame.from_dict(dict_out, orient="index").T
             list_out.append(df)
         try:
-            res_loop = get_request(url, auth_headers)
+            res_loop = get_request(url, auth_headers, log)
             res_data = res_loop.json()["results"]
             url = res_loop.json()["next"]
         except:
@@ -852,25 +846,38 @@ def get_scenario_url_data(
     return pd.concat(list_out)
 
 
-def get_request(url, headers):
+def get_request(url, headers, logger):
     """
     Wrapper for a GET request
 
     Handles throttle limits by using a timer
     """
     response = requests.get(url=url, headers=headers, verify=False)
-    if check_throttle_limit(response):
+    if check_throttle_limit(response, logger):
         response = requests.get(url=url, headers=headers, verify=False)
+
+    try:
+        response.raise_for_status()
+    except requests.RequestException as ex:
+        logger.write(
+            f"Candas: Error when making a request to '{url}'. " f"Exception: {ex}."
+        )
+        return
+
     return response
 
 
-def check_throttle_limit(response):
+def check_throttle_limit(response, logger):
     """
     Checks to see if a throttle limit has been hit, waits until throttle is lifted.
     """
     if response.status_code == 429:
         timer = int(response.headers.get("Retry-After"))
-        print(f"Please wait...Retrying request in {timer} seconds")
+        err_message = (
+            f"Request limit hit. Please wait...Retrying request in {timer} seconds."
+        )
+        print(err_message)
+        logger.write(err_message)
         time.sleep(timer)
         return True
     return False
@@ -900,7 +907,9 @@ def select_distinct(df, col):
     return d
 
 
-def create_drivers_dot(time_series, auth_headers, ticker, config=None, s3_client=None):
+def create_drivers_dot(
+    time_series, auth_headers, ticker, logger: LogFile, config=None, s3_client=None
+):
     if time_series == "":
         time_series = "net-revenue"
 
@@ -933,7 +942,7 @@ def create_drivers_dot(time_series, auth_headers, ticker, config=None, s3_client
         return path_name
     else:
         csin, version = get_company_info_from_ticker(
-            ticker, auth_headers, config.mds_host
+            ticker, auth_headers, logger, config.mds_host
         )
         url = (
             f"{config.mds_host}/api/equity-model-series/{csin}/equity-models/"
@@ -966,26 +975,16 @@ def get_model_url(ticker, wp_host=settings.WP_HOST):
 
 
 # retrieve latest equity model from API
-def get_model_info(
-    ticker, auth_headers, default_dir=settings.DEFAULT_DIR, mds_host=settings.MDS_HOST
-):
+def get_model_info(ticker, auth_headers, log: LogFile, mds_host=settings.MDS_HOST):
     """
     Retrieve latest equity model information and historical datapoints endpoint url
     """
-    log = LogFile(default_dir=default_dir)
     model_info = {}
 
     company_url = f"{mds_host}/{CSIN_URL.format(ticker=quote_plus(ticker))}"
 
-    response = requests.get(company_url, headers=auth_headers, verify=False)
-    try:
-        json = response.json().get("results")[0]
-    except:
-        log.write(
-            f"Candas: Error with {company_url}.  Perhaps you need the "
-            "correct Canalyst ticker including exchange."
-        )
-        return
+    company_response = get_request_json_content(company_url, auth_headers, log)
+    json = company_response.get("results")[0]
     csin = json.get("csin")
     company = json.get("company", {}).get("name")
     latest_version = (
@@ -995,12 +994,14 @@ def get_model_info(
         "earnings_update_type"
     )
 
-    periods_url = f"{mds_host}/{PERIODS_URL.format(csin=csin, version=latest_version)}"
-    periods_response = requests.get(periods_url, headers=auth_headers, verify=False)
-    periods_json = periods_response.json().get("results")
-    periods = [
-        period.get("name") for period in periods_json[:HISTORICAL_PERIODS_TO_FETCH]
-    ]
+    periods = get_historical_periods(
+        csin,
+        latest_version,
+        auth_headers,
+        mds_host,
+        log,
+        HISTORICAL_PERIODS_TO_FETCH,
+    )
 
     model_info[ticker] = (csin, company, latest_version, periods, earnings_update_type)
 
@@ -1012,6 +1013,22 @@ def get_model_info(
     return return_string, model_info
 
 
+def get_historical_periods(
+    csin,
+    version,
+    auth_headers,
+    mds_host,
+    log,
+    num_of_periods_to_fetch=HISTORICAL_PERIODS_TO_FETCH,
+):
+    periods_url = f"{mds_host}/{PERIODS_URL.format(csin=csin, version=version)}"
+    periods_response = get_request_json_content(periods_url, auth_headers, log)
+    periods_json = periods_response.get("results")
+    periods = [period.get("name") for period in periods_json[:num_of_periods_to_fetch]]
+
+    return periods
+
+
 def write_json(json_data, file_name):
     """
     Write JSON to file
@@ -1026,6 +1043,7 @@ def crawl_company_pages(
     key_name,
     next_name,
     auth_headers,
+    logger: LogFile,
     default_dir=settings.DEFAULT_DIR,
 ):
     """
@@ -1035,7 +1053,9 @@ def crawl_company_pages(
 
     page_number = 1
     while next_url is not None:
-        response, next_url = get_json(next_url, key_name, next_name, auth_headers)
+        response_content = get_request_json_content(next_url, auth_headers, logger)
+        response = response_content.get(key_name)
+        next_url = response_content.get(next_name)
         files_path = f"{default_dir}/DATA/{file_ticker}"
         os.makedirs(files_path, exist_ok=True)
         file_name = f"{files_path}/{page_number}.json"
@@ -1046,18 +1066,6 @@ def crawl_company_pages(
     # get api json data
 
 
-def get_json(url, key_name, next_name, auth_headers):
-    """
-    Make a GET request
-
-    Returns (response, next_url)
-    """
-    response = requests.get(url, headers=auth_headers, verify=False)
-    res = response.json().get(key_name)
-    next_url = response.json().get(next_name)
-    return res, next_url
-
-
 def read_json(ticker, default_dir=settings.DEFAULT_DIR):
     ticker = ticker.split(" ")[0]
     files_path = f"{default_dir}/DATA/{ticker}/"
@@ -1410,14 +1418,14 @@ def calendar_quarter(df, col, datetime=True):
     return df
 
 
-def get_drivers_from_api(mds_host, csin, model_version, api_headers):
+def get_drivers_from_api(mds_host, csin, model_version, api_headers, logger):
     """
     Returns a list of drivers for the model with the specified CSIN and model version
     """
     driver_list = []
 
     forecast_period_name = get_forecast_period_name(
-        mds_host, csin, model_version, api_headers
+        mds_host, csin, model_version, api_headers, logger
     )
 
     driver_url = "api/equity-model-series/{csin}/equity-models/{model_version}/forecast-periods/{period}/data-points/?is_driver=true&page_size=200"
@@ -1425,13 +1433,15 @@ def get_drivers_from_api(mds_host, csin, model_version, api_headers):
     next_url: Optional[str] = driver_endpoint
 
     while next_url is not None:
-        drivers, next_url = _get_drivers_from_next_url(next_url, api_headers)
+        json = get_request_json_content(next_url, headers=api_headers, logger=logger)
+        drivers = json.get("results")
+        next_url = json.get("next")
         driver_list.extend(drivers)
 
     return driver_list
 
 
-def get_forecast_period_name(mds_host, csin, model_version, api_headers):
+def get_forecast_period_name(mds_host, csin, model_version, api_headers, logger):
     """
     Retrieve the first forecast period name
     """
@@ -1445,26 +1455,20 @@ def get_forecast_period_name(mds_host, csin, model_version, api_headers):
         print(f"Candas: Error with getting forecast period.")
         return
 
-    forecast_period_name = response.json()["results"][0]["name"]
+    response_content = get_request_json_content(forecast_endpoint, api_headers, logger)
+
+    forecast_period_name = response_content["results"][0]["name"]
 
     return forecast_period_name
 
 
-def _get_drivers_from_next_url(url, api_headers):
+def get_request_json_content(url, headers, logger):
     """
-    Get drivers from specified url
+    Make a GET request to a URL that responds with JSON content
+
+    Returns the deserialized response content
     """
-    try:
-        response = requests.get(url, headers=api_headers)
-    except:
-        print(f"Candas: Error with getting drivers.")
-        return
-
-    response_json = response.json()
-    drivers = response_json["results"]
-    next_url = response_json["next"]
-
-    return drivers, next_url
+    return get_request(url, headers, logger).json()
 
 
 def get_chart_brand_config(
@@ -1546,9 +1550,7 @@ def _validate_colors(
     )
 
 
-def unstack_dataframe(
-    df, ticker="", time_series_name=[], period_type="period_end_date"
-):
+def unstack_dataframe(df, ticker="", time_series_name=[], period_type="period_name"):
     """
     A function to unstack a dataframe with period_name as index and either all or specific time series names as column headers.
 
@@ -1573,8 +1575,8 @@ def unstack_dataframe(
         df = df.loc[df["time_series_name"] == time_series_name]
 
     dx = (
-        df[["value", "time_series_name", period_type]]
-        .set_index([period_type, "time_series_name"])
+        df[["ticker", "value", "time_series_name", period_type]]
+        .set_index(["ticker", period_type, "time_series_name"])
         .unstack()
     )
     dx.columns = ["_".join(column) for column in dx.columns.to_flat_index()]
